package ca.ualberta.cs.poker.free.statistics;

import java.io.BufferedWriter;
import java.io.IOException;
import java.text.NumberFormat;
import java.util.Locale;
import java.util.Vector;


public class TournamentGrid {

	DataSet dataSet;
	RandomVariable[][] variables;
	Vector<String> players;
	boolean writeMean;
	boolean writePopulation;
	boolean writeEstimate;
	NumberFormat numberFormat;
	
	/**
	 * 
	 * Produce a N x N chart in text or HTML form to show a grid of RandomVariables (specificly
	 * its subclasses) and its values. It is assumed we want to show the RandomVariable's mean
	 * and standard deviation, and black out the diagonal since a player playing itself in a
	 * tournament is unlikely
	 * 
	 * @param data DataSet to produce the chart
	 * @param fractionDigits number of digits to show in the chart
	 * @param mean display the mean
	 * @param population include population std dev
	 * @param estimate include estimate std dev
	 */
	public TournamentGrid( DataSet data, int fractionDigits, boolean mean, boolean population, boolean estimate) {
		dataSet = data;
		players = data.getPlayers();
		variables = new RandomVariable[players.size()][players.size()];
		writeMean = mean;
		writePopulation = population;
		writeEstimate = estimate;
		
		numberFormat = NumberFormat.getNumberInstance(Locale.getDefault() );
		numberFormat.setMaximumFractionDigits( fractionDigits );
	}
	/**
	 * Constructor with common options, namely Mean +- EstimateStdDev
	 * 
	 * @param data
	 * @param fractionDigits
	 */
	public TournamentGrid( DataSet data, int fractionDigits) {
		this(data, fractionDigits, true, false, true );
	}
	
	
	
	/**
	 * Fill the array of variables that will be used for the tables with
	 * SmallBetPerHand variables. You could be fancy and do this stuff on the
	 * fly, but since each RandomVariable will have a different constructor this
	 * gets overkill really quick
	 * 
	 * A method for each type of RandomVariable will have to be created
	 * 
	 * firstHand and lastHand are parameters in the interest of future competitions
	 * where [0,999] might not be the default
	 * 
	 * @param firstHand
	 * @param lastHand
	 */
	public void fillSmallBetsPerHand( int firstHand, int lastHand ) {
		for( int i = 0; i < players.size(); i++ ) {
			for( int j = 0; j < players.size(); j++ ) {
				if ( i == j ) {
					variables[i][j] = null;
				} else {
					variables[i][j] = new SmallBetPerHandVariable(players.get(i),players.get(j), firstHand, lastHand, dataSet );
				}
			}
		}
	}
	
	/**
	 * write an HTML file to disk containing a header for the grid, plus a colored output
	 * determined by the constructor values
	 * 
	 * @param writer
	 * @throws IOException
	 */
	public void writeHTML(BufferedWriter writer) throws IOException {

		writer.write( "<h4> " );
		if ( writeMean ) writer.write( "Mean " );
		if( writePopulation) writer.write( " &plusmn; Population Standard Deviation");
		if( writeEstimate ) writer.write( " &plusmn; Estimate Standard Deviation");
		
		
		writer.write("<table style=\"color: black;\" align=\"center\" bgcolor=\"white\" border=\"1\">\n");

		// first column
		writer.write("<tbody><tr><td></td>\n");
		for (String player : players) {
			writer.write("<td>" + player + "</td>\n");
		}

		writer.write("</tr>");

		// each column
		for (int i = 0; i < players.size(); i++) {
			// each row
			writer.write("<td> " + players.get(i) + " </td>");

			for (int j = 0; j < players.size(); j++) {
				
				// fill player i vs player i with a black cell
				if (i == j) {
					writer.write("<td bgcolor=\"BLACK\"></td>");
				} else {
					// make the positive numbers green to indicate a win
					if( writeMean ) {
						if( getMeanValue(i, j) > 0 )
							writer.write( "<td bgcolor=\"GREEN\">" + numberFormat.format( getMeanValue(i, j) ) );
						else
							writer.write( "<td bgcolor=\"RED\">" + numberFormat.format( getMeanValue(i, j) ) );
					}
					
					if ( writePopulation ) writer.write( " &plusmn; " + numberFormat.format( getPopulationStdDev(i, j)));
					
					if( writeEstimate ) writer.write( " &plusmn; " + numberFormat.format( getEstimateStdDev(i, j)));
					
					writer.write( "</td>");
				}
			}
			writer.write("</tr>\n");

		}

		writer.write("</tbody> </table> ");
		writer.close();
	}
	
	
	public void writeTXT() {
		
		System.out.println( "START CHART");
		// write the top row
		for ( String player: players ) {
			System.out.print( player + " ");
		}
		System.out.print("\n");
		
		for( int i = 0; i < players.size(); i++ ) {
			System.out.print( "\n" + players.get(i) + " ");
			for( int j = 0; j < players.size(); j++ ) {
				if ( i == j ) {
					System.out.print( "XXX ");
				} else {
					System.out.print( getMeanValue(i, j) + " " );
				}
			}
			System.out.print("\n");
			
		}

		System.out.println( "\nEND CHART");

	}
	
	public double getMeanValue( int i, int j ){
		return variables[i][j].getSampleMean();
	}
	public double getPopulationVariance( int i, int j ){
		return variables[i][j].popVariance();
	}
	public double getPopulationStdDev( int i, int j ){
		return variables[i][j].popStdDev();
	}
	public double getEstimateVariance( int i, int j ){
		return variables[i][j].estVariance();
	}
	public double getEstimateStdDev( int i, int j ){
		return variables[i][j].estStdDev();
	}
	
	
	public void fillVariables() {
		
		variables = new RandomVariable[players.size()][players.size()];
		
		for( int i = 0; i < players.size(); i++ ) {
			for( int j = 0; j < players.size(); j++ ) {
				if ( i == j ) {
					variables[i][j] = null;
				} else {
					variables[i][j] = new SmallBetPerHandVariable(players.get(i),players.get(j), 0, 999, dataSet );
				}
			}
		}
	}

	
}
